/* fork.c - `pipe', `fork' and `exec' version server.
 * $Id: fork.c,v 1.9 2004/02/20 02:10:53 tagga Exp $
 *
 * Copyright (c) 2003-2004 TAGA Yoshitaka <tagga@tsuda.ac.jp>
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */
#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif /* HAVE_STDLIB_H */

#include <signal.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif /* HAVE_SYS_TYPES_H */
#ifdef HAVE_SYS_WAIT_H
#include <sys/wait.h>
#endif /* HAVE_SYS_WAIT_H */
#if !defined(HAVE_DUP2)
#  if defined(HAVE_FCNTL_H)
#    include <fcntl.h>
#  endif /* defined(HAVE_FCNTL_H) */
#endif /* !defined(HAVE_DUP2) */

#include "fork.h"
#if !defined(XERROR)
# error "XERROR must be defined."
#else
void XERROR();
#endif /* XERROR */

static void
sigterm(void)
{
  kill(0, SIGTERM);
}

FILE *
escm_popen(char * const argv[])
{
  int fd[2];
  pid_t pid;

  if (pipe(fd) < 0) XERROR("can't create a pipe");
  if ((pid = fork()) < 0)  XERROR("can't create a child process");
  if (pid > 0) { /* parent process */
    FILE *pipe;

    close(fd[0]); /* read */
    pipe = fdopen(fd[1], "w"); /* write */
    if (pipe == NULL) {
      kill(pid, SIGTERM);
      XERROR("can't open a pipe");
    }
    return pipe;
  } else { /* child process */
    if (atexit(sigterm))
      XERROR("can't terminate the parent process");

    close(fd[1]); /* write */
    /* connect fd[0] to the child process's stdin */
    escm_redirect(fileno(stdin), fd[0]);
    /* Invoke the interpreter */
    if (argv[0]) execvp(argv[0], argv);
    /* never reached if successful. */
    XERROR("can't invoke the interpreter");
    return NULL; /* dummy */
  }
}

int
escm_pclose(FILE *pipe)
{
  int status;

  fflush(pipe);
  fclose(pipe);
  /* wait till the child process exits. */
  wait(&status);
  if (WIFEXITED(status)) return WEXITSTATUS(status);
  else return -1;
}

void
escm_redirect(int from, int to)
{
  int ret;
#ifdef HAVE_DUP2
  ret = dup2(to, from);
#else /* not HAVE_DUP2 */
  close(from);
  ret = fcntl(to, F_DUPFD, from);
#endif /* HAVE_DUP2 */

  if (ret < 0) XERROR("can't redirect a file or stream");
  /* the value itself is ignored. */
}
/* end of fork.c */
